; Quick Tip
; ----------------------------------------------------------------------
; By Dan Morgan, Morgan Information Systems, 76164,2640
;
; For many applications I design a large number of reports based upon
; values present in "look-up help" tables.  For each report it is
; necessary to ask the user to provide the value(s) which will be used
; to get the records for the report.
;
; Rather than writing many separate procedures to accept and verify the
; values entered by the user I wrote the GetCode procedure.  The lookup
; tables all use a table structure in which the first field is named
; "Code" and is defined as "A3*" (alphanumeric and indexed).  The second
; field is called "Description".  The calling procedure contains the
; query used to create the set of records used in the report.  It
; provides values for Prompt.a, Table.a, and VV.s.  GetCode returns the
; value Code.a which is used in the report query.
;
; This routine could be beefed up by allowing the user to use "Escape"
; to back up from the "Custom" menu to the first menu and to verify that
; the user has not entered the same choice more than once into the
; "Custom" list.
; -------------------------------------------
; Requires display prompt in the variable Prompt.a
; Requires name of look-up table in the variable Table.a
; Requires screen row number in the variable VV.s
; Returns user choice in the variable Code.a
; -------------------------------------------
PROC GetCode(Prompt.a,Table.a,VV.s)
PRIVATE CodeArray,                   ; Array of codes
        Counter.s,                   ; Counter
        Custom.l,
        DescArray,                   ; Array of descriptions
        Display.a,                   ; Codes chosen display
        VV2.s,
        x,

Clear                                ; Clears the PAL canvas
MESSAGE "Please Wait ... "           ; For slower computers
Canvas OFF Cursor OFF                ; Freeze for speed-up & appearance

Code.a = ""
Custom.l = 0
VV2.s = VV.s + 2

VIEW Table.a                         ; If in a different directory
x = NRecords(Table.a)                ; Determine # of records in table

ARRAY CODEARRAY[x+2]                 ; Declare arrays equal to the
ARRAY DESCARRAY[x+2]                 ; number of codes + 2

Counter.s = 1                        ; Begin counting at "1"
MOVETO FIELD "Code"                  ; First field of look-up table
SCAN
   Counter.s = Counter.s + 1         ; Begins placing values in #2
   CodeArray[Counter.s] = []
ENDSCAN

Counter.s = 1                        ; Reset counter
MOVETO FIELD "DESCRIPTION"
SCAN
   Counter.s = Counter.s + 1         ; Begin placing values in #2
   DescArray[Counter.s] = []
ENDSCAN

CodeArray[1] = "ALL"                       ; Fill first element of array
DescArray[1] = "Include ALL Program Codes In This Report"
CodeArray[x+2] = "CUSTOM"                  ; Fill last element of array
DescArray[x+2] = "Enter A Custom Group Of Codes Separated By 'OR'"

@VV.s,01 ?? FORMAT("AC,W80",Prompt.a)      ; User defined prompt
PaintCanvas Fill " " 23,01,23,79           ; Clear please wait message
Canvas ON                                  ; Display the user prompt

While True                                 ; Enter closed loop
   SHOWARRAY CodeArray DescArray           ; Display ring menu
   TO Choice.a
   SWITCH
      CASE Choice.a = "ALL"      : Code.a = "" QuitLoop
      CASE Choice.a = "CUSTOM"   : CodeArray[1] = "COMPLETE"
                         DescArray[1] = "Custom List Complete"
                         CodeArray[x+2] = "CLEAR"
                         DescArray[x+2] = "Clear Choices, Start Again"
                         Custom.l = 1
      CASE Choice.a = "COMPLETE" : x = LEN(Code.a)
                  Code.a = SUBSTR(Code.a,1,x-3)
                  QuitLoop
      CASE Choice.a = "CLEAR" : PaintCanvas Fill " " VV2.s,10,VV2.s,79
                                Code.a = ""
      CASE Choice.a = "Esc"   : Loop
      OTHERWISE : If Custom.l = 1 Then
                     Code.a = Code.a + Choice.a + " OR "
                     x = LEN(Code.a)
                     Display.a = SUBSTR(Code.a,1,x-3)
                     @VV2.s,10 ?? Display.a
                  Else
                     Code.a = Choice.a
                     QuitLoop
                  EndIf
   ENDSWITCH
EndWhile

Return Code.a

ENDPROC
